/*
 * Copyright 2002-2005 The Apache Software Foundation.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
package net.didion.vfs.provider.crypt;

import javax.crypto.KeyGenerator;
import javax.crypto.Cipher;
import java.security.Key;
import java.security.Provider;
import java.security.Security;

public class CipherInfo {
    public static final String DEFAULT_PROVIDER = "SunJCE";
    public static final String DEFAULT_ALGORITHM = "AES";
    public static final int DEFAULT_KEY_SIZE = 128;

    private String providerName;
    private Provider provider;
    private String algorithm;
    private int keysize;
    private KeyProvider key;

    public CipherInfo() {
        this(null, null, -1, null);
    }

    public CipherInfo(
            String provider,
            String algorithm,
            int keysize,
            KeyProvider key) {
        this.providerName = (null == provider) ? DEFAULT_PROVIDER : provider;
        this.algorithm = (null == algorithm) ? DEFAULT_ALGORITHM : algorithm;
        this.keysize = (keysize <= 0) ? DEFAULT_KEY_SIZE : keysize;
        this.key = key;
    }

    public String getProviderName() {
        return providerName;
    }

    public String getAlgorithm() {
        return algorithm;
    }

    public int getKeysize() {
        return keysize;
    }

    private Provider getProvider() {
        if (null == this.provider) {
            this.provider = Security.getProvider(this.providerName);
            if (null == this.provider) {
                this.provider = Security.getProvider(DEFAULT_PROVIDER);
            }
        }
        return this.provider;
    }

    private Key getKey() throws Exception {
        if (null == this.key) {
            this.key = generateKey();
        }
        return key.getKey();
    }

    private KeyProvider generateKey() {
        KeyGenerator keygen;
        try {
            try {
                keygen = KeyGenerator.getInstance(getAlgorithm(), getProvider());
            } catch (Exception ex) {
                keygen = KeyGenerator.getInstance(getAlgorithm());
            }
            keygen.init(getKeysize());
            return new DefaultKeyProvider(keygen.generateKey());
        } catch (Exception ex) {
            throw new RuntimeException("Error generating key", ex);
        }
    }

    public Cipher createCipher(int mode) throws Exception {
        Cipher cipher;
        try {
            cipher = Cipher.getInstance(getAlgorithm(), getProvider());
        } catch (Exception ex) {
            cipher = Cipher.getInstance(getAlgorithm());
        }
        cipher.init(mode, getKey());
        return cipher;
    }
}